include("addresses.lua");
local windowList = findWindowList("Guild Wars 2");
if( #windowList == 0 ) then
	error("You need to run GW2 first!")
end
function getWin()
	if( __WIN == nil ) then
  		__WIN = windowList[1]
	end

	return __WIN;
end	

function getProc()
	if( __PROC == nil or not windowValid(__WIN) ) then
		if( __PROC ) then closeProcess(__PROC) end;
		__PROC = openProcess( findProcessByWindow(getWin()) );
	end

	return __PROC;
end	

function memoryReadRepeat(_type, proc, address, offset)
	local readfunc;
	local ptr = false;
	local val;

	if( type(proc) ~= "userdata" ) then
		error("Invalid proc", 2);
	end

	if( type(address) ~= "number" ) then
		error("Invalid address", 2);
	end

	if( _type == "int" ) then
		readfunc = memoryReadInt;
	elseif( _type == "uint" ) then
		readfunc = memoryReadUInt;
	elseif( _type == "float" ) then
		readfunc = memoryReadFloat;
	elseif( _type == "byte" ) then
		readfunc = memoryReadByte;
	elseif( _type == "string" ) then
		readfunc = memoryReadString;
	elseif( _type == "intptr" ) then
		readfunc = memoryReadIntPtr;
		ptr = true;
	elseif( _type == "uintptr" ) then
		readfunc = memoryReadUIntPtr;
		ptr = true;
	elseif( _type == "byteptr" ) then
		readfunc = memoryReadBytePtr;
		ptr = true;

	else
		return nil;
	end

	for i = 1, 10 do
		if( ptr ) then
			val = readfunc(proc, address, offset);
		else
			val = readfunc(proc, address);
		end

		if( val ~= nil ) then
			return val;
		end
	end

end	

function updatecoords()
	playerX = memoryReadFloat(getProc(), addresses.playerX) or 0
	playerZ = memoryReadFloat(getProc(), addresses.playerZ) or 0
	playerY = memoryReadFloat(getProc(), addresses.playerY) or 0
	local Vec1 = memoryReadFloat(getProc(), addresses.playerdir1) or 0
	local Vec2 = memoryReadFloat(getProc(), addresses.playerdir2) or 0
	playerDirection = (math.atan2(Vec2, Vec1) + math.pi) -- added pi to make it 0 to 2pi same as RoM
end
function updatehp()
	playerhp = memoryReadFloatPtr(getProc(), addresses.playerhp, addresses.playerhp_offset) or 0
	playermaxhp = memoryReadFloatPtr(getProc(), addresses.playerhp, addresses.playermaxhp_offset) or 0
end


function mouseclick(_x, _y, _wwide, _whigh, _type)
	if( foregroundWindow() ~= getWin() ) then
		print(language[139]);	-- D3 window has to be in the foreground
		return;
	end

	detach(); -- Remove attach bindings

	local wx,wy,wwide,whigh  = windowRect(getWin());
	local hf_x, hf_y;

	-- recalulate clickpoints depending from the actual D3 windows size
	-- only if we know the original windows size from the clickpoints
	if(_wwide  and _whigh) then
		hf_x = wwide * _x / _wwide;
		hf_y = whigh * _y / _whigh;
	else
		hf_x = _x;
		hf_y = _y;
	end;

	mouseSet(wx + hf_x, wy + hf_y);
	yrest(100);
	--print("clicking mouse ", wx + hf_x, wy + hf_y)
	mouseLClick();

	yrest(100);

	attach(getWin()); -- Re-attach bindings
end


function setwindow()
	unregisterTimer("timedSetWindowName");
	unregisterTimer("setwindow")
	local function _window()
		updatehp()
		setWindowName(getHwnd(),sprintf("Max HP: "..playermaxhp.." Actual HP: "..playerhp))
		--updatecoords()
		--setWindowName(getHwnd(),sprintf("direction : "..playerDirection))		
	end
	registerTimer("setwindow", secondsToTimer(1), _window);
end

function distance(x1, z1, y1, x2, z2, y2)
	if z2 == nil and y2 == nil then -- assume x1,z1,x2,z2 values (2 dimensional)
		z2 = x2
		x2 = y1
		y1 = nil
	end

	if( x1 == nil or z1 == nil or x2 == nil or z2 == nil ) then
		error("Error: nil value passed to distance()", 2);
	end

	if y1 == nil or y2 == nil then -- 2 dimensional calculation
		return math.sqrt( (z2-z1)*(z2-z1) + (x2-x1)*(x2-x1) );
	else -- 3 dimensional calculation
		return math.sqrt( (z2-z1)*(z2-z1) + (x2-x1)*(x2-x1) + (y2-y1)*(y2-y1) );
	end
end
function angleDifference(angle1, angle2)
  if( math.abs(angle2 - angle1) > math.pi ) then
    return (math.pi * 2) - math.abs(angle2 - angle1);
  else
    return math.abs(angle2 - angle1);
  end
end

function moveto(waypoint, range)
	local playerLastDistImprove
	local function passed_point(lastpos, point)
		point.X = tonumber(point.X)
		point.Z = tonumber(point.Z)

		local posbuffer = 50

		local passed = true
		if lastpos.X < point.X and playerX < point.X - posbuffer then
			return false
		end
		if lastpos.X > point.X and playerX > point.X + posbuffer then
			return false
		end
		if lastpos.Z < point.Z and playerZ < point.Z - posbuffer then
			return false
		end
		if lastpos.Z > point.Z and playerZ > point.Z + posbuffer then
			return false
		end
		return true
	end

	updatecoords()
	local lastpos = {X=playerX, Z=playerZ, Y=playerY}

	local angle = math.atan2(waypoint.Z - playerZ, waypoint.X - playerX);
	--if angle > math.pi*0.5 then angle = angle - (math.pi * 1.5) else angle = angle + (math.pi * 0.5) end
	local yangle = 0
	if waypoint.Y ~= nil then
		yangle = math.atan2(waypoint.Y - playerY, ((waypoint.X - playerX)^2 + (waypoint.Z - playerZ)^2)^.5 );
	end
	local angleDif = angleDifference(angle, playerDirection);
	local startTime = os.time();


	-- no active turning and moving if wander and radius = 0
	-- self direction has values from 0 til Pi and -Pi til 0
	-- angle has values from 0 til 2*Pi
	if(__WPL:getMode()   == "wander"  and
	   __WPL:getRadius() == 0     )   then
		if( playerDirection < 0 ) then
			angle = (math.pi * 2) - math.abs(playerDirection);
		else
			angle = playerDirection;
		end;

		-- we will not move back to WP if wander and radius = 0
		-- so one can move the character manual and use the bot only as fight support
		-- there we set the WP to the actual player position
		waypoint.Z = playerZ;
		waypoint.X = playerX;

	end;


	-- If more than X degrees off, correct before moving.
	local rotateStartTime = os.time();
	while( angleDif > math.rad(30) ) do

		if( os.difftime(os.time(), rotateStartTime) > 3.0 ) then
			-- Sometimes both left and right rotate get stuck down.
			-- Press them both to make sure they are fully released.
			keyboardPress(settings.hotkeys.ROTATE_LEFT.key);
			keyboardPress(settings.hotkeys.ROTATE_RIGHT.key);

			-- we seem to have been distracted, take a step back.
			keyboardPress(settings.hotkeys.MOVE_BACKWARD.key);
			rotateStartTime = os.time();
		end

		if( angleDifference(angle, playerDirection + 0.01) < angleDif ) then
			-- rotate left
			keyboardRelease( settings.hotkeys.ROTATE_RIGHT.key );
			keyboardHold( settings.hotkeys.ROTATE_LEFT.key );
		else
			-- rotate right
			keyboardRelease( settings.hotkeys.ROTATE_LEFT.key );
			keyboardHold( settings.hotkeys.ROTATE_RIGHT.key );
		end

		yrest(50);
		updatecoords()
		angleDif = angleDifference(angle, playerDirection);
	end
	keyboardRelease( settings.hotkeys.ROTATE_LEFT.key );
	keyboardRelease( settings.hotkeys.ROTATE_RIGHT.key );

	-- Direction ok, start moving forward
	local success, failreason = true, WF_NONE;
	local dist = distance(playerX, playerZ, playerY, waypoint.X, waypoint.Z, waypoint.Y);
	local lastDist = dist;
	playerLastDistImprove = os.time();	-- global, because we reset it whil skill use

	local turning = false

	local loopstart
	local loopduration = 100 -- The duration we want the loop to take
	local successdist = 100
	repeat
		loopstart = os.clock()

		-- We're still making progress
		if( dist < lastDist ) then
			playerLastDistImprove = os.time();
			lastDist = dist;
		elseif(  dist > lastDist + 800 ) then
			-- Make sure we didn't pass it up
			printf(language[29]);
			success = false;
			failreason = WF_DIST;
			break;
		end;

		if( os.difftime(os.time(), playerLastDistImprove) > 3 ) then
			-- We haven't improved for 3 seconds, assume stuck
			success = false;
			failreason = WF_STUCK;
			break;
		end

		dist = distance(playerX, playerZ, playerY, waypoint.X, waypoint.Z, waypoint.Y);
		angle = math.atan2(waypoint.Z - playerZ, waypoint.X - playerX);

		-- Check if within range if range specified
		if range and range > dist then
			-- within range
			break
		end

		-- Check if past waypoint
		if passed_point(lastpos, waypoint) then
		   -- waypoint reached
		   break
		end

		-- Check if close to waypoint.
		if dist < successdist then
			break
		end

		lastpos = {X=playerX, Z=playerZ, Y=playerY}

		if waypoint.Y ~= nil then
			yangle = math.atan2(waypoint.Y - playerY, ((waypoint.X - playerX)^2 + (waypoint.Z - playerZ)^2)^.5 );
		end
		angleDif = angleDifference(angle, playerDirection);

		-- Continue to make sure we're facing the right direction
			if( angleDif > math.rad(15) ) then
				--keyboardRelease( settings.hotkeys.MOVE_FORWARD.key );
				--keyboardRelease( settings.hotkeys.MOVE_BACKWARD.key );

				if( angleDifference(angle, playerDirection + 0.01) < angleDif ) then
						keyboardRelease( settings.hotkeys.ROTATE_RIGHT.key );
						keyboardHold( settings.hotkeys.ROTATE_LEFT.key );
						yrest(100);
				else
						keyboardRelease( settings.hotkeys.ROTATE_LEFT.key );
						keyboardHold( settings.hotkeys.ROTATE_RIGHT.key );
						yrest(100);
				end
				turning = true
			elseif( angleDif > math.rad(1) ) then

				--faceDirection(angle, yangle);
				keyboardRelease( settings.hotkeys.ROTATE_LEFT.key );
				keyboardRelease( settings.hotkeys.ROTATE_RIGHT.key );
				keyboardHold( settings.hotkeys.MOVE_FORWARD.key );
				turning = false
			else
				keyboardHold( settings.hotkeys.MOVE_FORWARD.key );
			end


		--keyboardHold( settings.hotkeys.MOVE_FORWARD.key );
		local pausetime = loopduration - (os.clock() - loopstart) -- minus the time already elapsed.
		if pausetime < 1 then pausetime = 1 end
		yrest(pausetime);
		updatecoords()
		waypoint:update();

	until false

	keyboardRelease( settings.hotkeys.MOVE_FORWARD.key );

	keyboardRelease( settings.hotkeys.ROTATE_LEFT.key );
	keyboardRelease( settings.hotkeys.ROTATE_RIGHT.key );

	return success, failreason;
end